@@ -48,16 +48,27 @@ static int ip_converter(PyObject *obj, struct sockaddr_storage *ip_address);
 #endif
 
 static int Reader_init(PyObject *self, PyObject *args, PyObject *kwds) {
-    char *filename;
+    PyObject *filepath = NULL;
     int mode = 0;
 
     static char *kwlist[] = {"database", "mode", NULL};
-    if (!PyArg_ParseTupleAndKeywords(
-            args, kwds, "s|i", kwlist, &filename, &mode)) {
+    if (!PyArg_ParseTupleAndKeywords(args,
+                                     kwds,
+                                     "O&|i",
+                                     kwlist,
+                                     PyUnicode_FSConverter,
+                                     &filepath,
+                                     &mode)) {
+        return -1;
+    }
+
+    char *filename = PyBytes_AS_STRING(filepath);
+    if (filename == NULL) {
         return -1;
     }
 
     if (mode != 0 && mode != 1) {
+        Py_XDECREF(filepath);
         PyErr_Format(
             PyExc_ValueError,
             "Unsupported open mode (%i). Only "
@@ -67,26 +78,29 @@ static int Reader_init(PyObject *self, PyObject *args, PyObject *kwds) {
     }
 
     if (0 != access(filename, R_OK)) {
-        PyErr_Format(PyExc_FileNotFoundError,
-                     "No such file or directory: '%s'",
-                     filename);
+
+        PyErr_SetFromErrnoWithFilenameObject(PyExc_OSError, filepath);
+        Py_XDECREF(filepath);
         return -1;
     }
 
     MMDB_s *mmdb = (MMDB_s *)malloc(sizeof(MMDB_s));
     if (NULL == mmdb) {
+        Py_XDECREF(filepath);
         PyErr_NoMemory();
         return -1;
     }
 
     Reader_obj *mmdb_obj = (Reader_obj *)self;
     if (!mmdb_obj) {
+        Py_XDECREF(filepath);
         free(mmdb);
         PyErr_NoMemory();
         return -1;
     }
 
     uint16_t status = MMDB_open(filename, MMDB_MODE_MMAP, mmdb);
+    Py_XDECREF(filepath);
 
     if (MMDB_SUCCESS != status) {
         free(mmdb);
@@ -723,7 +737,7 @@ PyMODINIT_FUNC PyInit_extension(void) {
     if (PyType_Ready(&Metadata_Type)) {
         return NULL;
     }
-    PyModule_AddObject(m, "extension", (PyObject *)&Metadata_Type);
+    PyModule_AddObject(m, "Metadata", (PyObject *)&Metadata_Type);
 
     PyObject *error_mod = PyImport_ImportModule("maxminddb.errors");
     if (error_mod == NULL) {
