@@ -19,7 +19,7 @@ use Test::MaxMind::DB::Common::Util qw( standard_test_metadata );
 my $Dir = dirname( abs_path($0) );
 
 sub main {
-    my @sizes      = ( 24,        28, 32 );
+    my @sizes = ( 24, 28, 32 );
     my @ipv4_range = ( '1.1.1.1', '1.1.1.32' );
 
     my @ipv4_subnets = Net::Works::Network->range_as_subnets(@ipv4_range);
@@ -71,6 +71,7 @@ sub main {
     }
 
     write_decoder_test_db();
+    write_pointer_decoder_test_db();
     write_deeply_nested_structures_db();
 
     write_geoip2_dbs();
@@ -231,23 +232,7 @@ sub write_test_db {
     );
 
     sub write_decoder_test_db {
-        my $writer = MaxMind::DB::Writer::Tree->new(
-            ip_version    => 6,
-            record_size   => 24,
-            database_type => 'MaxMind DB Decoder Test',
-            languages     => ['en'],
-            description   => {
-                en =>
-                    'MaxMind DB Decoder Test database - contains every MaxMind DB data type',
-            },
-            alias_ipv6_to_ipv4       => 1,
-            remove_reserved_networks => 0,
-            map_key_type_callback    => sub {
-                my $key = $_[0];
-                $key =~ s/X$//;
-                return $key eq 'array' ? [ 'array', 'uint32' ] : $key;
-            },
-        );
+        my $writer = _decoder_writer();
 
         my @subnets
             = map { Net::Works::Network->new_from_string( string => $_ ) }
@@ -285,6 +270,70 @@ sub write_test_db {
 
         return;
     }
+
+    sub write_pointer_decoder_test_db {
+
+        # We want to create a database where most values are pointers
+        no warnings 'redefine';
+        local *MaxMind::DB::Writer::Serializer::_should_cache_value
+            = sub { 1 };
+        my $writer = _decoder_writer();
+
+        # We add these slightly different records so that we end up with
+        # pointers for the individual values in the maps, not just pointers
+        # to the map
+        $writer->insert_network(
+            '1.0.0.0/32',
+            {
+                %all_types,
+                booleanX => 0,
+                arrayX   => [ 1, 2, 3, 4, ],
+                mapXX    => {
+                    utf8_stringX => 'hello',
+                    arrayX       => [ 7, 8, 9, 10 ],
+                    booleanX     => 0,
+                },
+            },
+        );
+
+        $writer->insert_network(
+            '1.1.1.0/32',
+            {
+                %all_types,
+
+                # This has to be 0 rather than 1 as otherwise the buggy
+                # Perl writer will think it is the same as an uint32 value of
+                # 1 and make a pointer to a value of a different type.
+                boolean => 0,
+            },
+        );
+
+        open my $fh, '>', "$Dir/MaxMind-DB-test-pointer-decoder.mmdb";
+        $writer->write_tree($fh);
+        close $fh;
+
+        return;
+    }
+
+    sub _decoder_writer {
+        return MaxMind::DB::Writer::Tree->new(
+            ip_version    => 6,
+            record_size   => 24,
+            database_type => 'MaxMind DB Decoder Test',
+            languages     => ['en'],
+            description   => {
+                en =>
+                    'MaxMind DB Decoder Test database - contains every MaxMind DB data type',
+            },
+            alias_ipv6_to_ipv4       => 1,
+            remove_reserved_networks => 0,
+            map_key_type_callback    => sub {
+                my $key = $_[0];
+                $key =~ s/X*$//;
+                return $key eq 'array' ? [ 'array', 'uint32' ] : $key;
+            },
+        );
+    }
 }
 
 {
@@ -362,6 +411,8 @@ sub write_geoip2_dbs {
         ['GeoIP2-Static-IP-Score'],
         ['GeoIP2-User-Count'],
         ['GeoLite2-ASN'],
+        ['GeoLite2-City'],
+        ['GeoLite2-Country'],
         );
 }
 
@@ -427,6 +478,7 @@ sub _universal_map_key_type_callback {
         is_in_european_union           => 'boolean',
         is_legitimate_proxy            => 'boolean',
         is_public_proxy                => 'boolean',
+        is_residential_proxy           => 'boolean',
         is_satellite_provider          => 'boolean',
         is_tor_exit_node               => 'boolean',
         iso_code                       => 'utf8_string',
@@ -605,7 +657,7 @@ sub write_test_serialization_data {
     $serializer->store_data( map => { long_key  => 'long_value1' } );
     $serializer->store_data( map => { long_key2 => 'long_value2' } );
 
-    open my $fh, '>', 'maps-with-pointers.raw';
+    open my $fh, '>', "$Dir/maps-with-pointers.raw";
     print {$fh} ${ $serializer->buffer() }
         or die "Cannot write to maps-with-pointers.raw: $!";
     close $fh;
@@ -631,7 +683,7 @@ sub write_db_with_metadata_pointers {
 
     _populate_all_networks( $writer, {} );
 
-    open my $fh, '>', 'MaxMind-DB-test-metadata-pointers.mmdb';
+    open my $fh, '>', "$Dir/MaxMind-DB-test-metadata-pointers.mmdb";
 
     $writer->write_tree($fh);
 
