@@ -3,6 +3,7 @@
 
 import ipaddress
 import os
+import pathlib
 import threading
 import unittest
 import unittest.mock as mock
@@ -211,6 +212,15 @@ class BaseTestReader(object):
         self.assertEqual(1329227995784915872903807060280344576, record["uint128"])
         reader.close()
 
+    def test_metadata_pointers(self):
+        with open_database(
+            "tests/data/test-data/MaxMind-DB-test-metadata-pointers.mmdb", self.mode
+        ) as reader:
+            self.assertEqual(
+                "Lots of pointers in metadata",
+                reader.metadata().database_type,
+            )
+
     def test_no_ipv4_search_tree(self):
         reader = open_database(
             "tests/data/test-data/MaxMind-DB-no-ipv4-search-tree.mmdb", self.mode
@@ -233,9 +243,15 @@ class BaseTestReader(object):
             reader.get(self.ipf("2001::"))
         reader.close()
 
+    def test_opening_path(self):
+        with open_database(
+            pathlib.Path("tests/data/test-data/MaxMind-DB-test-decoder.mmdb"), self.mode
+        ) as reader:
+            self.assertEqual(reader.metadata().database_type, "MaxMind DB Decoder Test")
+
     def test_no_extension_exception(self):
-        real_extension = maxminddb.extension
-        maxminddb.extension = None
+        real_extension = maxminddb._extension
+        maxminddb._extension = None
         with self.assertRaisesRegex(
             ValueError,
             "MODE_MMAP_EXT requires the maxminddb.extension module to be available",
@@ -243,7 +259,7 @@ class BaseTestReader(object):
             open_database(
                 "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", MODE_MMAP_EXT
             )
-        maxminddb.extension = real_extension
+        maxminddb._extension = real_extension
 
     def test_broken_database(self):
         reader = open_database(
